@@ -30,12 +30,16 @@ function progress_update($pos, $max)
 function export_mailbox($mbox, $filename)
 {
 	global $IMAP;
-	
-	$IMAP->set_mailbox($mbox);
-	
+
+	$IMAP->set_folder($mbox);
+
+    $index = $IMAP->index($mbox, null, 'ASC');
+    $count = $index->countMessages();
+    $index = $index->get();
+
 	vputs("Getting message list of {$mbox}...");
-	vputs($IMAP->messagecount()." messages\n");
-	
+	vputs("$count messages\n");
+
 	if ($filename)
 	{
 		if (!($out = fopen($filename, 'w')))
@@ -47,21 +51,20 @@ function export_mailbox($mbox, $filename)
 	}
 	else
 		$out = STDOUT;
-	
-	for ($count = $IMAP->messagecount(), $i=1; $i <= $count; $i++)
+
+	for ($i = 0; $i < $count; $i++)
 	{
-		$headers = $IMAP->get_headers($i, null, false);
-		$from = current($IMAP->decode_address_list($headers->from, 1, false));
-		
+		$headers = $IMAP->get_message_headers($index[$i]);
+		$from = current(rcube_mime::decode_address_list($headers->from, 1, false));
+
 		fwrite($out, sprintf("From %s %s UID %d\n", $from['mailto'], $headers->date, $headers->uid));
-		fwrite($out, $IMAP->conn->fetchPartHeader($mbox, $i));
-		fwrite($out, $IMAP->conn->handlePartBody($mbox, $i));
+		fwrite($out, $IMAP->print_raw_body($headers->uid));
 		fwrite($out, "\n\n\n");
-		
-		progress_update($i, $count);
+
+		progress_update($i+1, $count);
 	}
 	vputs("\ncomplete.\n");
-	
+
 	if ($filename)
 		fclose($out);
 }
@@ -114,9 +117,9 @@ $IMAP = new rcube_imap(null);
 if ($IMAP->connect($host, $args['user'], $args['pass'], $imap_port, $imap_ssl))
 {
 	vputs("IMAP login successful.\n");
-	
+
 	$filename = null;
-	$mailboxes = $args['mbox'] == '*' ? $IMAP->list_mailboxes(null) : array($args['mbox']);
+	$mailboxes = $args['mbox'] == '*' ? $IMAP->list_folders(null) : array($args['mbox']);
 
 	foreach ($mailboxes as $mbox)
 	{
@@ -124,7 +127,7 @@ if ($IMAP->connect($host, $args['user'], $args['pass'], $imap_port, $imap_ssl))
 			$filename = preg_replace('/\.[a-z0-9]{3,4}$/i', '', $args['file']) . asciiwords($mbox) . '.mbox';
 		else if ($args['mbox'] == '*')
 			$filename = asciiwords($mbox) . '.mbox';
-			
+
 		if ($args['mbox'] == '*' && in_array(strtolower($mbox), array('junk','spam','trash')))
 			continue;
 
